import * as _tanstack_react_query from '@tanstack/react-query';
import { UseMutationOptions, UseQueryResult, UseQueryOptions, MutationOptions } from '@tanstack/react-query';
import * as _supabase_postgrest_js from '@supabase/postgrest-js';
import { PostgrestQueryBuilder, PostgrestError as PostgrestError$1, PostgrestSingleResponse, PostgrestMaybeSingleResponse, PostgrestResponse } from '@supabase/postgrest-js';
import { GetResult } from '@supabase/postgrest-js/dist/module/select-query-parser';
import { GenericSchema, GenericTable } from '@supabase/postgrest-js/dist/module/types';
import { PostgrestMutatorOpts } from '@supabase-cache-helpers/postgrest-mutate';
import { PostgrestError, RealtimePostgresChangesPayload, SupabaseClient, RealtimePostgresChangesFilter, REALTIME_POSTGRES_CHANGES_LISTEN_EVENT, RealtimeChannel } from '@supabase/supabase-js';
import { AnyPostgrestResponse } from '@supabase-cache-helpers/postgrest-shared';

type Operation = 'Insert' | 'UpdateOne' | 'Upsert' | 'DeleteOne';
type GetInputType<T extends GenericTable, O extends Operation> = O extends 'DeleteOne' ? Partial<T['Row']> : O extends 'Insert' | 'Upsert' ? T['Insert'][] : O extends 'UpdateOne' ? T['Update'] : never;
type GetReturnType<S extends GenericSchema, T extends GenericTable, O extends Operation, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>> = O extends 'UpdateOne' ? R | null : O extends 'DeleteOne' ? R | null : O extends 'Insert' | 'Upsert' ? R[] | null : never;
type UsePostgrestMutationOpts<S extends GenericSchema, T extends GenericTable, O extends Operation, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>> = PostgrestMutatorOpts<T['Row']> & UseMutationOptions<GetReturnType<S, T, O, Q, R> | null, PostgrestError, GetInputType<T, O>> & {
    disableAutoQuery?: boolean;
};

declare function useDeleteMutation<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: (Q extends '*' ? "'*' is not allowed" : Q) | null, opts?: Omit<UsePostgrestMutationOpts<S, T, 'DeleteOne', Q, R>, 'mutationFn'>): _tanstack_react_query.UseMutationResult<NonNullable<R> | null, _supabase_postgrest_js.PostgrestError, Partial<T["Row"]>, unknown>;

declare function useInsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: (Q extends '*' ? "'*' is not allowed" : Q) | null, opts?: Omit<UsePostgrestMutationOpts<S, T, 'Insert', Q, R>, 'mutationFn'>): _tanstack_react_query.UseMutationResult<(R extends false | "" | 0 | null | undefined ? never : R)[] | null, _supabase_postgrest_js.PostgrestError, T["Insert"][], unknown>;

declare function useUpdateMutation<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: (Q extends '*' ? "'*' is not allowed" : Q) | null, opts?: Omit<UsePostgrestMutationOpts<S, T, 'UpdateOne', Q, R>, 'mutationFn'>): _tanstack_react_query.UseMutationResult<NonNullable<R> | null, _supabase_postgrest_js.PostgrestError, T["Update"], unknown>;

declare function useUpsertMutation<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(qb: PostgrestQueryBuilder<S, T>, primaryKeys: (keyof T['Row'])[], query?: (Q extends '*' ? "'*' is not allowed" : Q) | null, opts?: Omit<UsePostgrestMutationOpts<S, T, 'Upsert', Q, R>, 'mutationFn'>): _tanstack_react_query.UseMutationResult<(R extends false | "" | 0 | null | undefined ? never : R)[] | null, _supabase_postgrest_js.PostgrestError, T["Insert"][], unknown>;

type UseQuerySingleReturn<Result> = Omit<UseQueryResult<PostgrestSingleResponse<Result>['data'], PostgrestError$1>, 'refetch'> & Pick<UseQueryResult<PostgrestSingleResponse<Result>, PostgrestError$1>, 'refetch'> & Pick<PostgrestSingleResponse<Result>, 'count'>;
type UseQueryMaybeSingleReturn<Result> = Omit<UseQueryResult<PostgrestMaybeSingleResponse<Result>['data'], PostgrestError$1>, 'refetch'> & Pick<UseQueryResult<PostgrestMaybeSingleResponse<Result>, PostgrestError$1>, 'refetch'> & Pick<PostgrestMaybeSingleResponse<Result>, 'count'>;
type UseQueryReturn<Result> = Omit<UseQueryResult<PostgrestResponse<Result>['data'], PostgrestError$1>, 'refetch'> & Pick<UseQueryResult<PostgrestResponse<Result>, PostgrestError$1>, 'refetch'> & Pick<PostgrestResponse<Result>, 'count'>;
type UseQueryAnyReturn<Result> = Omit<UseQueryResult<AnyPostgrestResponse<Result>['data'], PostgrestError$1>, 'refetch'> & Pick<UseQueryResult<AnyPostgrestResponse<Result>, PostgrestError$1>, 'refetch'> & Pick<AnyPostgrestResponse<Result>, 'count'>;
/**
 * Perform a postgrest query
 * @param query
 * @param config
 */
declare function useQuery<Result>(query: PromiseLike<PostgrestSingleResponse<Result>>, config?: Omit<UseQueryOptions<PostgrestSingleResponse<Result>, PostgrestError$1>, 'queryKey' | 'queryFn'>): UseQuerySingleReturn<Result>;
declare function useQuery<Result>(query: PromiseLike<PostgrestMaybeSingleResponse<Result>>, config?: Omit<UseQueryOptions<PostgrestMaybeSingleResponse<Result>, PostgrestError$1>, 'queryKey' | 'queryFn'>): UseQueryMaybeSingleReturn<Result>;
declare function useQuery<Result>(query: PromiseLike<PostgrestResponse<Result>>, config?: Omit<UseQueryOptions<PostgrestResponse<Result>, PostgrestError$1>, 'queryKey' | 'queryFn'>): UseQueryReturn<Result>;

type UseSubscriptionQueryOpts<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>> = PostgrestMutatorOpts<T['Row']> & MutationOptions & {
    callback?: (event: RealtimePostgresChangesPayload<T['Row']> & {
        data: T['Row'] | R;
    }) => void | Promise<void>;
};
declare function useSubscriptionQuery<S extends GenericSchema, T extends GenericTable, Q extends string = '*', R = GetResult<S, T['Row'], Q extends '*' ? '*' : Q>>(client: SupabaseClient | null, channelName: string, filter: Omit<RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, 'table'> & {
    table: string;
}, primaryKeys: (keyof T['Row'])[], query?: Q extends '*' ? "'*' is not allowed" : Q | null, opts?: UseSubscriptionQueryOpts<S, T, Q, R>): {
    channel: RealtimeChannel | undefined;
    status: string | undefined;
};

type UseSubscriptionOpts<T extends GenericTable> = PostgrestMutatorOpts<T['Row']> & MutationOptions & {
    callback?: (event: RealtimePostgresChangesPayload<T['Row']>) => void | Promise<void>;
};
declare function useSubscription<T extends GenericTable>(channel: RealtimeChannel | null, filter: Omit<RealtimePostgresChangesFilter<`${REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.ALL}`>, 'table'> & {
    table: string;
}, primaryKeys: (keyof T['Row'])[], opts?: UseSubscriptionOpts<T>): {
    status: string | undefined;
};

export { UseQueryAnyReturn, UseSubscriptionOpts, UseSubscriptionQueryOpts, useDeleteMutation, useInsertMutation, useQuery, useSubscription, useSubscriptionQuery, useUpdateMutation, useUpsertMutation };
